/**
 *  '$RCSfile: ElementNode.java,v $'
 *    Purpose: A Class that represents an XML element and its contents,
 *             and can build itself from a database connection
 *  Copyright: 2000 Regents of the University of California and the
 *             National Center for Ecological Analysis and Synthesis
 *    Authors: Matt Jones
 *    Release: 1.0.3
 *
 *   '$Author: jones $'
 *     '$Date: 2001/01/18 19:51:59 $'
 * '$Revision: 1.24 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package edu.ucsb.nceas.metacat;

import java.sql.*;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

/**
 * A Class that represents an XML element and its contents,
 * and can build itself from a database connection
 */
public class ElementNode extends BasicNode {

  /** 
   * Construct a new ElementNode instance, and recursively create its children
   *
   * @param nodeRecordList the nodedata to use to initialize, which is a
   *        TreeSet of NodeRecord objects
   * @param nodeid the identifier for the node to be created
   */
  public ElementNode (TreeSet nodeRecordList, long nodeid) {

    MetaCatUtil util = new MetaCatUtil();

    // Step through all of the node records we were given
    Iterator it = nodeRecordList.iterator();
    while (it.hasNext()) {
      NodeRecord currentNode = (NodeRecord)it.next();
      if (currentNode.nodeid == nodeid) {
        util.debugMessage("Got Node ID: " + currentNode.nodeid +
                          " (" + currentNode.parentnodeid +
                          ", " + currentNode.nodeindex + 
                          ", " + currentNode.nodetype + ")");
        // Process the current node
        setNodeType(currentNode.nodetype);
        setNodeID(currentNode.nodeid);
        setParentID(currentNode.parentnodeid);
        setTagName(currentNode.nodename);
      } else {
        // Process the children nodes
        if (currentNode.parentnodeid == getNodeID()) {
          util.debugMessage("  Processing child: " + currentNode.nodeid +
                          " (" + currentNode.parentnodeid +
                          ", " + currentNode.nodeindex + 
                          ", " + currentNode.nodetype + ")");

          if ((currentNode.nodetype).equals("ELEMENT")) {
            util.debugMessage("Creating child node: " + currentNode.nodeid);
            ElementNode child = new ElementNode(nodeRecordList,
                                                currentNode.nodeid);
            addChildNode(child);
          } else if (currentNode.nodetype.equals("ATTRIBUTE")) {
            setAttribute(currentNode.nodename,currentNode.nodedata);
          } else if (currentNode.nodetype.equals("TEXT")) {
            TextNode child = new TextNode(currentNode.nodeid,
                                          currentNode.parentnodeid,
                                          currentNode.nodedata);
            addChildNode(child);
          } else if (currentNode.nodetype.equals("COMMENT")) {
            CommentNode child = new CommentNode(currentNode.nodeid,
                                                currentNode.parentnodeid,
                                                currentNode.nodedata);
            addChildNode(child);
          } else if (currentNode.nodetype.equals("PI")) {
            PINode child = new PINode(currentNode.nodeid,
                                      currentNode.parentnodeid,
                                      currentNode.nodename,
                                      currentNode.nodedata);
            addChildNode(child);
          }

        } else {
          util.debugMessage("  Discarding child: " + currentNode.nodeid +
                          " (" + currentNode.parentnodeid +
                          ", " + currentNode.nodeindex +
                          ", " + currentNode.nodetype + ")");
        }
      }
    }
  }

  /** 
   * String representation for display purposes (recursively descends through
   * children to create an XML subtree)
   */
  public String toString () {

    StringBuffer value = new StringBuffer();
    String nodetype = getNodeType();

    if (nodetype.equals("ELEMENT")) {
      value.append('<');
      value.append(getTagName());
      value.append(getAttributes().toString());
      value.append('>');
    } 

    // Process children recursively here
    BasicNode child = null;
    Enumeration e = getChildren();
    while (e.hasMoreElements()) {
      child = (BasicNode)e.nextElement(); 
      value.append(child);
    }

    if (nodetype.equals("ELEMENT")) {
      value.append("</");
      value.append(getTagName());
      value.append('>');
    }

    return value.toString();
  }
}
